=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-12-18 04:08:46 +0000
+++ bzrlib/builtins.py	2006-01-01 00:53:37 +0000
@@ -2006,3 +2006,4 @@
 # we do need to load at least some information about them to know of 
 # aliases.
 from bzrlib.conflicts import cmd_resolve, cmd_conflicts, restore
+from bzrlib.changeset.commands import cmd_changeset

=== renamed file 'bzrlib/changeset/old/commands.py' => 'bzrlib/changeset/commands.py'
--- bzrlib/changeset/old/commands.py	2005-11-19 04:22:51 +0000
+++ bzrlib/changeset/commands.py	2006-01-01 00:53:37 +0000
@@ -11,6 +11,9 @@
 from bzrlib.branch import Branch
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.option import Option
+from bzrlib.revision import (common_ancestor, MultipleRevisionSources,
+                             get_intervening_revisions, NULL_REVISION)
+import bzrlib.errors as errors
 
 class cmd_send_changeset(Command):
     """Send a bundled up changset via mail.
@@ -50,110 +53,78 @@
 
         send_changeset(b, revision, to, message, file)
 
+
 class cmd_changeset(Command):
     """Generate a bundled up changeset.
 
     This changeset contains all of the meta-information of a
     diff, rather than just containing the patch information.
 
-    BASE - This is the target tree with which you want to merge.
-           It will be used as the base for all patches. Anyone
-           wanting to merge the changeset will be required to have BASE
-    TARGET - This is the final revision which is desired to be in the
-             changeset. It defaults to the last committed revision. './@'
-    STARTING-REV-ID - All revisions between STARTING-REV and TARGET will
-                      be bundled up in the changeset. By default this is
-                      chosen as the merge root.
-                      (NOT Implemented yet)
-
-
-    If --verbose, renames will be given as an 'add + delete' style patch.
-    If --revision is given, it has several states:
-        --revision A..B A is chosen as the base and B is chosen as the target
-        --revision A    A is chosen as the target, and the base is it's primary parent
-        --revision ..B  B is chosen as the target, and the base is it's primary parent
-        --revision A..  ???
-    --message will replace the global message with the one supplied. Though this
-        will not be saved in the final revision messages.
+    bzr cset
+        - Changeset for the last commit
+    bzr cset BASE
+        - Changeset to apply the current tree into BASE
+    bzr cset --revision A
+        - Changeset for revision A
+    bzr cset --revision A..B
+        - Changeset to transform A into B
+    bzr cset --revision A..B BASE
+        - Changeset to transform revision A of BASE into revision B
+          of the local tree
     """
-    takes_options = ['verbose', 'revision', 'message']
-    takes_args = ['base?', 'target?', 'starting-rev-id?']
+    takes_options = ['verbose', 'revision']
+    takes_args = ['base?']
     aliases = ['cset']
 
-    def run(self, base=None, target=None, starting_rev_id=None,
-            verbose=False, revision=None, message=None):
-        from bzrlib.commands import parse_spec
-        from bzrlib.errors import BzrCommandError
+    def run(self, base=None, revision=None):
         from bzrlib import user_encoding
-        import gen_changeset
-        import codecs
-
-        if revision is not None:
-            if (target is not None or base is not None):
-                raise BzrCommandError('--revision superceeds base and target')
-            if len(revision) == 1:
-                target_info = revision[0]
-                base_info = None
-            elif len(revision) == 2:
-                target_info = revision[1]
-                base_info = revision[0]
-            else:
-                raise BzrCommandError('--revision can take at most 2 arguments')
-
-            target_branch, relpath = Branch.open_containing('.')
-            target_revno, target_rev_id = target_info.in_history(target_branch)
-            base_branch = target_branch
-            if base_info is not None:
-                base_revno, base_rev_id = base_info.in_history(target_branch)
-            else:
-                target_rev = target_branch.get_revision(target_rev_id)
-                if hasattr(target_rev, 'parent_ids'):
-                    base_rev_id = target_rev.parent_ids[0]
-                else:
-                    base_rev_id = target_rev.parents[0].revision_id
-        else:
-            if target is None:
-                target = './@'
-            b_target_path, target_revno = parse_spec(target)
-            target_branch, relpath = Branch.open_containing(b_target_path)
-            if target_revno is None or target_revno == -1:
-                if hasattr(target_branch, 'last_patch'):
-                    target_rev_id = target_branch.last_patch()
-                else:
-                    target_rev_id = target_branch.last_revision()
-            else:
-                target_rev_id = target_branch.get_rev_id(target_revno)
-
-            if base is None:
-                base_branch = target_branch
-                target_rev = target_branch.get_revision(target_rev_id)
-                if hasattr(target_rev, 'parent_ids'):
-                    base_rev_id = target_rev.parent_ids[0]
-                else:
-                    base_rev_id = target_rev.parents[0].revision_id
-            else:
-                base_path, base_revno = parse_spec(base)
-                base_branch, relpath = Branch.open_containing(base_path)
-                if base_revno is None or base_revno == -1:
-                    if hasattr(target_branch, 'last_patch'):
-                        base_rev_id = target_branch.last_patch()
-                    else:
-                        base_rev_id = target_branch.last_revision()
-                else:
-                    base_rev_id = base_branch.get_rev_id(base_revno)
-
-        # outf = codecs.getwriter(user_encoding)(sys.stdout,
-        #         errors='replace')
-
-        if starting_rev_id is not None:
-            raise BzrCommandError('Specifying the STARTING-REV-ID'
-                    ' not yet supported')
-
-        gen_changeset.show_changeset(base_branch, base_rev_id,
-                target_branch, target_rev_id,
-                starting_rev_id,
-                to_file=sys.stdout, include_full_diff=verbose,
-                message=message)
+        from bzrlib.changeset.serializer import write
+        from bzrlib.fetch import fetch
+
+        if base is None:
+            base_branch = None
+        else:
+            base_branch = Branch.open(base)
+
+        # We don't want to lock the same branch across
+        # 2 different branches
+        target_branch = Branch.open_containing(u'.')[0]
+        if base_branch is not None and target_branch.base == base_branch.base:
+            base_branch = None
+
+        base_revision = None
+        if revision is None:
+            target_revision = target_branch.last_revision()
+            if base_branch is not None:
+                base_revision = base_branch.last_revision()
+        elif len(revision) == 1:
+            target_revision = revision[0].in_history(target_branch).rev_id
+            if base_branch is not None:
+                base_revision = base_branch.last_revision()
+        elif len(revision) == 2:
+            target_revision = revision[0].in_history(target_branch).rev_id
+            if base_branch is not None:
+                base_revision = revision[1].in_history(base_branch).rev_id
+            else:
+                base_revision = revision[1].in_history(target_branch).rev_id
+        else:
+            raise errors.BzrCommandError('--revision takes 1 or 2 parameters')
+
+        if base_revision is None:
+            rev = target_branch.get_revision(target_revision)
+            if rev.parent_ids:
+                base_revision = rev.parent_ids[0]
+            else:
+                base_revision = NULL_REVISION
+
+        if base_branch is not None:
+            fetch(target_branch, base_branch, base_revision)
+            del base_branch
+        revision_id_list = get_intervening_revisions(target_revision, base_revision,
+                target_branch, target_branch.revision_history())
+                
+        write(target_branch, revision_id_list, sys.stdout)
+
 
 class cmd_verify_changeset(Command):
     """Read a written changeset, and make sure it is valid.
@@ -203,6 +174,7 @@
         apply_changeset.apply_changeset(b, f, reverse=reverse,
                 auto_commit=auto_commit)
 
+
 register_command(cmd_changeset)
 register_command(cmd_verify_changeset)
 register_command(cmd_apply_changeset)

=== modified file 'bzrlib/changeset/serializer/__init__.py'
--- bzrlib/changeset/serializer/__init__.py	2005-11-20 01:37:41 +0000
+++ bzrlib/changeset/serializer/__init__.py	2006-01-01 00:53:37 +0000
@@ -189,7 +189,7 @@
         """Read the rest of the changesets from the supplied file.
 
         :param f: The file to read from
-        :return: A list of changesets
+        :return: A list of changeset trees
         """
         raise NotImplementedError
 

